@@ -108,7 +108,7 @@ rocfft_plan make_plan(ROCFFT_LIB libhandle, const fft_params& params)
 {
     auto procfft_setup = (decltype(&rocfft_setup))rocfft_lib_symbol(libhandle, "rocfft_setup");
     if(procfft_setup == NULL)
-        exit(1);
+        throw rocfft_runtime_error("rocfft_setup failed");
     auto procfft_plan_description_create
         = (decltype(&rocfft_plan_description_create))rocfft_lib_symbol(
             libhandle, "rocfft_plan_description_create");
@@ -280,9 +280,16 @@ int main(int argc, char* argv[])
     // Control output verbosity:
     int verbose{};
 
+    // number of GPUs to use:
+    int ngpus{};
+
     // hip Device number for running tests:
     int deviceId{};
 
+    // Ignore runtime failures.
+    // eg: hipMalloc failing when there isn't enough free vram.
+    bool ignore_hip_runtime_failures{true};
+
     // Number of performance trial samples:
     int ntrial{};
 
@@ -298,6 +305,10 @@ int main(int argc, char* argv[])
     // FFT parameters:
     fft_params params;
 
+    // input/output FFT grids
+    std::vector<unsigned int> ingrid;
+    std::vector<unsigned int> outgrid;
+
     // Token string to fully specify fft params.
     std::string token;
 
@@ -307,7 +318,7 @@ int main(int argc, char* argv[])
     // FIXME: version needs to be implemented
     app.add_flag("--version",
                  "Print queryable version information from the rocfft library and exit");
-    app.add_option("--reverse", reverse, "Load libs in forward and reverse order")->default_val(1);
+    app.add_flag("--reverse", reverse, "Load libs in forward and reverse order")->default_val(1);
     app.add_option(
            "--sequence", test_sequence, "Test sequence:\n0) random\n1) alternating\n2) sequential")
         ->default_val(0);
@@ -348,6 +359,21 @@ int main(int argc, char* argv[])
     CLI::Option* opt_length
         = non_token->add_option("--length", params.length, "Lengths")->required()->expected(1, 3);
 
+    non_token->add_option("--ngpus", ngpus, "Number of GPUs to use")
+        ->default_val(1)
+        ->check(CLI::NonNegativeNumber);
+
+    // define multi-GPU grids for FFT computation,
+    CLI::Option* opt_ingrid
+        = non_token->add_option("--ingrid", ingrid, "Single-process grid of GPUs at input")
+              ->expected(1, 3)
+              ->needs("--ngpus");
+
+    CLI::Option* opt_outgrid
+        = non_token->add_option("--outgrid", outgrid, "Single-process grid of GPUs at output")
+              ->expected(1, 3)
+              ->needs("--ngpus");
+
     non_token
         ->add_option("-b, --batchSize",
                      params.nbatch,
@@ -364,6 +390,10 @@ int main(int argc, char* argv[])
     CLI::Option* opt_ioffset = non_token->add_option("--ioffset", params.ioffset, "Input offset");
     CLI::Option* opt_ooffset = non_token->add_option("--ooffset", params.ooffset, "Output offset");
 
+    app.add_flag("--ignore_runtime_failures,!--no-ignore_runtime_failures",
+                 ignore_hip_runtime_failures,
+                 "Ignore hip runtime failures");
+
     app.add_option("--device", deviceId, "Select a specific device id")->default_val(0);
     app.add_option("--verbose", verbose, "Control output verbosity")->default_val(0);
     app.add_option("-N, --ntrial", ntrial, "Trial size for the problem")
@@ -371,14 +401,13 @@ int main(int argc, char* argv[])
         ->each([&](const std::string& val) {
             std::cout << "Running profile with " << val << " samples\n";
         });
-
+    // Default value is set in fft_params.h based on if device-side PRNG was enabled.
     app.add_option("-g, --inputGen",
                    params.igen,
                    "Input data generation:\n0) PRNG sequence (device)\n"
                    "1) PRNG sequence (host)\n"
                    "2) linearly-spaced sequence (device)\n"
-                   "3) linearly-spaced sequence (host)")
-        ->default_val(fft_input_random_generator_device);
+                   "3) linearly-spaced sequence (host)");
     app.add_option("--isize", params.isize, "Logical size of input buffer");
     app.add_option("--osize", params.osize, "Logical size of output buffer");
     app.add_option("--scalefactor", params.scale_factor, "Scale factor to apply to output");
@@ -399,7 +428,7 @@ int main(int argc, char* argv[])
         if(!std::filesystem::exists(lib_string))
         {
             std::cerr << "Error: lib " << lib_string << " does not exist\n";
-            exit(1);
+            return EXIT_FAILURE;
         }
     }
 
@@ -414,11 +443,38 @@ int main(int argc, char* argv[])
         catch(...)
         {
             std::cout << "Unable to parse token." << std::endl;
-            return 1;
+            return EXIT_FAILURE;
         }
     }
     else
     {
+
+        if(ngpus > 1)
+        {
+            // set default GPU grids in case none were given
+            params.set_default_grid(ngpus, ingrid, outgrid);
+
+            // split the problem among ngpus
+            params.mp_lib = fft_params::fft_mp_lib_none;
+
+            int localDeviceCount = 0;
+            if(hipGetDeviceCount(&localDeviceCount) != hipSuccess)
+            {
+                throw std::runtime_error("hipGetDeviceCount failed");
+            }
+
+            // start with all-ones in grids
+            std::vector<unsigned int> input_grid(params.length.size() + 1, 1);
+            std::vector<unsigned int> output_grid(params.length.size() + 1, 1);
+
+            // create input and output grids and distribute it according to user requirements
+            std::copy(ingrid.begin(), ingrid.end(), input_grid.begin() + 1);
+            std::copy(outgrid.begin(), outgrid.end(), output_grid.begin() + 1);
+
+            params.distribute_input(localDeviceCount, input_grid);
+            params.distribute_output(localDeviceCount, output_grid);
+        }
+
         if(*opt_not_in_place)
         {
             std::cout << "out-of-place\n";
@@ -465,6 +521,21 @@ int main(int argc, char* argv[])
                 std::cout << " " << i;
             std::cout << "\n";
         }
+        if(*opt_ingrid || !ingrid.empty())
+        {
+            std::cout << "input  grid:";
+            for(auto& i : ingrid)
+                std::cout << " " << i;
+            std::cout << "\n";
+        }
+
+        if(*opt_outgrid || !outgrid.empty())
+        {
+            std::cout << "output grid:";
+            for(auto& i : outgrid)
+                std::cout << " " << i;
+            std::cout << "\n";
+        }
     }
     std::cout << std::flush;
 
@@ -475,7 +546,7 @@ int main(int argc, char* argv[])
 
     if(!params.valid(verbose))
     {
-        throw std::runtime_error("Invalid parameters, add --verbose=1 for detail");
+        throw rocfft_runtime_error("Invalid parameters, add --verbose=1 for detail");
     }
 
     std::cout << "Token: " << params.token() << std::endl;
@@ -487,7 +558,14 @@ int main(int argc, char* argv[])
     // Check free and total available memory:
     size_t free  = 0;
     size_t total = 0;
-    HIP_V_THROW(hipMemGetInfo(&free, &total), "hipMemGetInfo failed");
+    try
+    {
+        HIP_V_THROW(hipMemGetInfo(&free, &total), "hipMemGetInfo failed");
+    }
+    catch(rocfft_hip_runtime_error)
+    {
+        return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+    }
 
     const auto raw_vram_footprint
         = params.fft_params_vram_footprint() + twiddle_table_vram_footprint(params);
@@ -498,93 +576,43 @@ int main(int argc, char* argv[])
         return EXIT_SUCCESS;
     }
 
-    const auto vram_footprint = params.vram_footprint();
-    if(!vram_fits_problem(vram_footprint, free))
-    {
-        std::cout << "SKIPPED: Problem size (" << vram_footprint
-                  << ") raw data too large for device.\n";
-        return EXIT_SUCCESS;
-    }
-
     // GPU input buffer:
-    auto                ibuffer_sizes = params.ibuffer_sizes();
-    std::vector<gpubuf> ibuffer(ibuffer_sizes.size());
-    std::vector<void*>  pibuffer(ibuffer_sizes.size());
-    for(unsigned int i = 0; i < ibuffer.size(); ++i)
-    {
-        HIP_V_THROW(ibuffer[i].alloc(ibuffer_sizes[i]), "Creating input Buffer failed");
-        pibuffer[i] = ibuffer[i].data();
-    }
-
-    // CPU input buffer
+    std::vector<gpubuf> ibuffer;
+    std::vector<void*>  pibuffer;
+    // CPU-side input buffer
     std::vector<hostbuf> ibuffer_cpu;
 
-    auto is_device_gen = (params.igen == fft_input_generator_device
-                          || params.igen == fft_input_random_generator_device);
-    auto is_host_gen   = (params.igen == fft_input_generator_host
+    auto is_host_gen = (params.igen == fft_input_generator_host
                         || params.igen == fft_input_random_generator_host);
 
-    if(is_device_gen)
-    {
-        // Input data:
-        params.compute_input(ibuffer);
+    auto ibricks = get_input_bricks(params);
+    auto obricks = get_output_bricks(params);
 
-        if(verbose > 1)
-        {
-            // Copy input to CPU
-            ibuffer_cpu = allocate_host_buffer(params.precision, params.itype, params.isize);
-            for(unsigned int idx = 0; idx < ibuffer.size(); ++idx)
-            {
-                HIP_V_THROW(hipMemcpy(ibuffer_cpu.at(idx).data(),
-                                      ibuffer[idx].data(),
-                                      ibuffer_sizes[idx],
-                                      hipMemcpyDeviceToHost),
-                            "hipMemcpy failed");
-            }
-
-            std::cout << "GPU input:\n";
-            params.print_ibuffer(ibuffer_cpu);
-        }
-    }
-
-    if(is_host_gen)
-    {
-        // Input data:
-        ibuffer_cpu = allocate_host_buffer(params.precision, params.itype, params.isize);
-        params.compute_input(ibuffer_cpu);
-
-        if(verbose > 1)
-        {
-            std::cout << "GPU input:\n";
-            params.print_ibuffer(ibuffer_cpu);
-        }
-
-        for(unsigned int idx = 0; idx < ibuffer_cpu.size(); ++idx)
-        {
-            HIP_V_THROW(hipMemcpy(pibuffer[idx],
-                                  ibuffer_cpu[idx].data(),
-                                  ibuffer_cpu[idx].size(),
-                                  hipMemcpyHostToDevice),
-                        "hipMemcpy failed");
-        }
-    }
-
-    // GPU output buffer:
     std::vector<gpubuf>  obuffer_data;
-    std::vector<gpubuf>* obuffer = &obuffer_data;
-    if(params.placement == fft_placement_inplace)
+    std::vector<gpubuf>* obuffer = nullptr;
+    alloc_bench_bricks(
+        params, ibricks, obricks, ibuffer, obuffer_data, obuffer, ibuffer_cpu, is_host_gen);
+    init_bench_input(params, ibricks, ibuffer, ibuffer_cpu, is_host_gen);
+
+    for(unsigned int i = 0; i < ibuffer.size(); ++i)
     {
-        obuffer = &ibuffer;
+        pibuffer.push_back(ibuffer[i].data());
     }
-    else
+
+    // print input if requested
+    if(verbose > 1)
     {
-        auto obuffer_sizes = params.obuffer_sizes();
-        obuffer_data.resize(obuffer_sizes.size());
-        for(unsigned int i = 0; i < obuffer_data.size(); ++i)
+        if(is_host_gen)
         {
-            HIP_V_THROW(obuffer_data[i].alloc(obuffer_sizes[i]), "Creating output Buffer failed");
+            // data is already on host
+            params.print_ibuffer(ibuffer_cpu);
+        }
+        else
+        {
+            print_device_buffer(params, ibuffer, true);
         }
     }
+
     std::vector<void*> pobuffer(obuffer->size());
     for(unsigned int i = 0; i < obuffer->size(); ++i)
     {
@@ -609,15 +637,14 @@ int main(int argc, char* argv[])
     for(size_t ridx = 0; ridx < ntrial_runs.size(); ++ridx)
     {
 
-        std::vector<size_t> timeindex;
+        std::vector<std::pair<size_t, std::string>> index_lib_string;
         for(size_t i = 0; i < lib_strings.size(); ++i)
         {
-            timeindex.push_back(i);
+            index_lib_string.push_back(std::make_pair(i, lib_strings[i]));
         }
         if(ridx == 1)
         {
-            std::reverse(lib_strings.begin(), lib_strings.end());
-            std::reverse(timeindex.begin(), timeindex.end());
+            std::reverse(index_lib_string.begin(), index_lib_string.end());
         }
 
         // Create the handles to the libs and the associated fft plans.
@@ -644,10 +671,26 @@ int main(int argc, char* argv[])
         }
 
         std::cout << "Work buffer size: " << wbuffer_size << std::endl;
+
+        if(!vram_fits_problem(raw_vram_footprint + wbuffer_size, free))
+        {
+            std::cout << "SKIPPED: Problem size (" << raw_vram_footprint << " + " << +wbuffer_size
+                      << " = " << raw_vram_footprint + wbuffer_size
+                      << " ) data too large for device.\n";
+            return EXIT_SUCCESS;
+        }
+
         gpubuf wbuffer;
         if(wbuffer_size)
         {
-            HIP_V_THROW(wbuffer.alloc(wbuffer_size), "Creating intermediate Buffer failed");
+            try
+            {
+                HIP_V_THROW(wbuffer.alloc(wbuffer_size), "Creating intermediate Buffer failed");
+            }
+            catch(rocfft_hip_runtime_error)
+            {
+                return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+            }
         }
 
         // Associate the work buffer to the individual libraries:
@@ -659,10 +702,17 @@ int main(int argc, char* argv[])
         // Run the plan using its associated rocFFT library:
         for(unsigned int idx = 0; idx < handle.size(); ++idx)
         {
-            run_plan(handle[idx], plan[idx], info[idx], pibuffer.data(), pobuffer.data());
+            try
+            {
+                run_plan(handle[idx], plan[idx], info[idx], pibuffer.data(), pobuffer.data());
+            }
+            catch(rocfft_hip_runtime_error)
+            {
+                return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+            }
         }
 
-        std::vector<int> testcase(ntrial_runs[ridx] * lib_strings.size());
+        std::vector<int> testcase(ntrial_runs[ridx] * index_lib_string.size());
 
         switch(test_sequence)
         {
@@ -671,9 +721,10 @@ int main(int argc, char* argv[])
             // Random order:
             for(int itrial = 0; itrial < ntrial_runs[ridx]; ++itrial)
             {
-                for(size_t ilib = 0; ilib < lib_strings.size(); ++ilib)
+                for(size_t ilib = 0; ilib < index_lib_string.size(); ++ilib)
                 {
-                    testcase[lib_strings.size() * itrial + ilib] = ilib;
+
+                    testcase[index_lib_string.size() * itrial + ilib] = ilib;
                 }
             }
             std::random_device rd;
@@ -685,9 +736,9 @@ int main(int argc, char* argv[])
             // Alternating order:
             for(int itrial = 0; itrial < ntrial_runs[ridx]; ++itrial)
             {
-                for(size_t ilib = 0; ilib < lib_strings.size(); ++ilib)
+                for(size_t ilib = 0; ilib < index_lib_string.size(); ++ilib)
                 {
-                    testcase[lib_strings.size() * itrial + ilib] = ilib;
+                    testcase[index_lib_string.size() * itrial + ilib] = ilib;
                 }
             }
             break;
@@ -695,7 +746,7 @@ int main(int argc, char* argv[])
             // Sequential order:
             for(int itrial = 0; itrial < ntrial_runs[ridx]; ++itrial)
             {
-                for(size_t ilib = 0; ilib < lib_strings.size(); ++ilib)
+                for(size_t ilib = 0; ilib < index_lib_string.size(); ++ilib)
                 {
                     testcase[ilib * ntrial + itrial] = ilib;
                 }
@@ -721,39 +772,61 @@ int main(int argc, char* argv[])
 
             if(verbose > 3)
             {
-                std::cout << "running test case " << tidx << "\n";
+                std::cout << "running test case " << tidx << " with lib "
+                          << index_lib_string[tidx].second << "\n";
             }
 
-            if(is_device_gen)
-            {
+#ifdef USE_HIPRAND
+            if(!is_host_gen)
                 params.compute_input(ibuffer);
-            }
+#endif
             if(is_host_gen)
             {
                 for(unsigned int bidx = 0; bidx < ibuffer_cpu.size(); ++bidx)
                 {
-                    HIP_V_THROW(hipMemcpy(pibuffer[bidx],
-                                          ibuffer_cpu[bidx].data(),
-                                          ibuffer_cpu[bidx].size(),
-                                          hipMemcpyHostToDevice),
-                                "hipMemcpy failed");
+                    try
+                    {
+                        HIP_V_THROW(hipMemcpy(pibuffer[bidx],
+                                              ibuffer_cpu[bidx].data(),
+                                              ibuffer_cpu[bidx].size(),
+                                              hipMemcpyHostToDevice),
+                                    "hipMemcpy failed");
+                    }
+                    catch(rocfft_hip_runtime_error)
+                    {
+                        return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+                    }
                 }
             }
 
             // Run the plan using its associated rocFFT library:
-            time[tidx].push_back(
-                run_plan(handle[tidx], plan[tidx], info[tidx], pibuffer.data(), pobuffer.data()));
+            try
+            {
+                time[tidx].push_back(run_plan(
+                    handle[tidx], plan[tidx], info[tidx], pibuffer.data(), pobuffer.data()));
+            }
+            catch(rocfft_hip_runtime_error)
+            {
+                return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+            }
 
             if(verbose > 2)
             {
                 auto output = allocate_host_buffer(params.precision, params.otype, params.osize);
                 for(unsigned int iout = 0; iout < output.size(); ++iout)
                 {
-                    HIP_V_THROW(hipMemcpy(output[iout].data(),
-                                          pobuffer[iout],
-                                          output[iout].size(),
-                                          hipMemcpyDeviceToHost),
-                                "hipMemcpy failed");
+                    try
+                    {
+                        HIP_V_THROW(hipMemcpy(output[iout].data(),
+                                              pobuffer[iout],
+                                              output[iout].size(),
+                                              hipMemcpyDeviceToHost),
+                                    "hipMemcpy failed");
+                    }
+                    catch(rocfft_hip_runtime_error)
+                    {
+                        return ignore_hip_runtime_failures ? EXIT_SUCCESS : EXIT_FAILURE;
+                    }
                 }
                 std::cout << "GPU output:\n";
                 params.print_obuffer(output);
