@@ -40,8 +40,8 @@ __host__ __device__ Tdata load_callback(Tdata* input, size_t offset, void* cbdat
     }
 }
 
-__device__ auto load_callback_dev_half           = load_callback<_Float16>;
-__device__ auto load_callback_dev_complex_half   = load_callback<rocfft_complex<_Float16>>;
+__device__ auto load_callback_dev_half           = load_callback<rocfft_fp16>;
+__device__ auto load_callback_dev_complex_half   = load_callback<rocfft_complex<rocfft_fp16>>;
 __device__ auto load_callback_dev_float          = load_callback<float>;
 __device__ auto load_callback_dev_complex_float  = load_callback<rocfft_complex<float>>;
 __device__ auto load_callback_dev_double         = load_callback<double>;
@@ -66,9 +66,9 @@ __host__ __device__ Tdata
 }
 
 __device__ auto load_callback_round_trip_inverse_dev_half
-    = load_callback_round_trip_inverse<_Float16>;
+    = load_callback_round_trip_inverse<rocfft_fp16>;
 __device__ auto load_callback_round_trip_inverse_dev_complex_half
-    = load_callback_round_trip_inverse<rocfft_complex<_Float16>>;
+    = load_callback_round_trip_inverse<rocfft_complex<rocfft_fp16>>;
 __device__ auto load_callback_round_trip_inverse_dev_float
     = load_callback_round_trip_inverse<float>;
 __device__ auto load_callback_round_trip_inverse_dev_complex_float
@@ -218,8 +218,8 @@ __host__ __device__ static void
     }
     // otherwise, wrong base address passed, just don't write
 }
-__device__ auto store_callback_dev_half           = store_callback<_Float16>;
-__device__ auto store_callback_dev_complex_half   = store_callback<rocfft_complex<_Float16>>;
+__device__ auto store_callback_dev_half           = store_callback<rocfft_fp16>;
+__device__ auto store_callback_dev_complex_half   = store_callback<rocfft_complex<rocfft_fp16>>;
 __device__ auto store_callback_dev_float          = store_callback<float>;
 __device__ auto store_callback_dev_complex_float  = store_callback<rocfft_complex<float>>;
 __device__ auto store_callback_dev_double         = store_callback<double>;
@@ -230,7 +230,7 @@ __host__ __device__ static void store_callback_round_trip_inverse(
     Tdata* output, size_t offset, Tdata element, void* cbdata, void* sharedMem)
 {
     auto testdata = static_cast<callback_test_data*>(cbdata);
-    // add scalar to each element
+    // divide each element by scalar
     if(output == testdata->base)
     {
         output[offset] = element / testdata->scalar;
@@ -238,9 +238,9 @@ __host__ __device__ static void store_callback_round_trip_inverse(
     // otherwise, wrong base address passed, just don't write
 }
 __device__ auto store_callback_round_trip_inverse_dev_half
-    = store_callback_round_trip_inverse<_Float16>;
+    = store_callback_round_trip_inverse<rocfft_fp16>;
 __device__ auto store_callback_round_trip_inverse_dev_complex_half
-    = store_callback_round_trip_inverse<rocfft_complex<_Float16>>;
+    = store_callback_round_trip_inverse<rocfft_complex<rocfft_fp16>>;
 __device__ auto store_callback_round_trip_inverse_dev_float
     = store_callback_round_trip_inverse<float>;
 __device__ auto store_callback_round_trip_inverse_dev_complex_float
@@ -397,10 +397,11 @@ void apply_store_callback(const fft_params& params, std::vector<hostbuf>& output
         {
         case fft_precision_half:
         {
-            const size_t elem_size = sizeof(rocfft_complex<_Float16>);
+            const size_t elem_size = sizeof(rocfft_complex<rocfft_fp16>);
             const size_t num_elems = output.front().size() / elem_size;
 
-            auto output_begin = reinterpret_cast<rocfft_complex<_Float16>*>(output.front().data());
+            auto output_begin
+                = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(output.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 auto& element = output_begin[i];
@@ -454,12 +455,12 @@ void apply_store_callback(const fft_params& params, std::vector<hostbuf>& output
         {
         case fft_precision_half:
         {
-            const size_t elem_size = sizeof(rocfft_complex<_Float16>);
+            const size_t elem_size = sizeof(rocfft_complex<rocfft_fp16>);
             for(auto& buf : output)
             {
                 const size_t num_elems = buf.size() / elem_size;
 
-                auto output_begin = reinterpret_cast<rocfft_complex<_Float16>*>(buf.data());
+                auto output_begin = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(buf.data());
                 for(size_t i = 0; i < num_elems; ++i)
                 {
                     auto& element = output_begin[i];
@@ -512,10 +513,10 @@ void apply_store_callback(const fft_params& params, std::vector<hostbuf>& output
         {
         case fft_precision_half:
         {
-            const size_t elem_size = sizeof(_Float16);
+            const size_t elem_size = sizeof(rocfft_fp16);
             const size_t num_elems = output.front().size() / elem_size;
 
-            auto output_begin = reinterpret_cast<_Float16*>(output.front().data());
+            auto output_begin = reinterpret_cast<rocfft_fp16*>(output.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 auto& element = output_begin[i];
@@ -588,10 +589,10 @@ void apply_load_callback(const fft_params& params, std::vector<hostbuf>& input)
         {
         case fft_precision_half:
         {
-            const size_t elem_size = sizeof(rocfft_complex<_Float16>);
+            const size_t elem_size = sizeof(rocfft_complex<rocfft_fp16>);
             const size_t num_elems = input.front().size() / elem_size;
 
-            auto input_begin = reinterpret_cast<rocfft_complex<_Float16>*>(input.front().data());
+            auto input_begin = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(input.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
@@ -631,10 +632,10 @@ void apply_load_callback(const fft_params& params, std::vector<hostbuf>& input)
         {
         case fft_precision_half:
         {
-            const size_t elem_size = sizeof(_Float16);
+            const size_t elem_size = sizeof(rocfft_fp16);
             const size_t num_elems = input.front().size() / elem_size;
 
-            auto input_begin = reinterpret_cast<_Float16*>(input.front().data());
+            auto input_begin = reinterpret_cast<rocfft_fp16*>(input.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
