@@ -28,6 +28,7 @@
 
 #include "../../shared/fftw_transform.h"
 #include "../../shared/hip_object_wrapper.h"
+#include "../../shared/params_gen.h"
 #include "../../shared/rocfft_params.h"
 #include "rocfft/rocfft.h"
 
@@ -83,6 +84,15 @@ struct is_hip_complex<rocfft_complex<double>>
     static const bool value = true;
 };
 
+// ASAN introduces some problems with mixing library and client
+// callbacks, so skip these tests if it's enabled
+#ifdef ADDRESS_SANITIZER
+#define TEST_CALLBACK_CHECK_ASAN \
+    GTEST_SKIP() << "mixed library/client callbacks not supported for ASAN";
+#else
+#define TEST_CALLBACK_CHECK_ASAN
+#endif
+
 // -------------------------------------
 // test callbacks struct
 // -------------------------------------
@@ -400,8 +410,16 @@ struct Test_Callback
 // complex data inputs and having only a load callback set.
 // -------------------------------------------------------------------
 
+#ifndef WIN32
 TEST(rocfft_UnitTest, default_load_callback_complex_single)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(256,
                        1,
                        rocfft_transform_type_complex_forward,
@@ -412,6 +430,13 @@ TEST(rocfft_UnitTest, default_load_callback_complex_single)
 
 TEST(rocfft_UnitTest, default_load_callback_complex_double)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(512,
                        1,
                        rocfft_transform_type_complex_forward,
@@ -422,6 +447,13 @@ TEST(rocfft_UnitTest, default_load_callback_complex_double)
 
 TEST(rocfft_UnitTest, default_load_callback_real_single)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(1024,
                        1,
                        rocfft_transform_type_real_forward,
@@ -432,6 +464,13 @@ TEST(rocfft_UnitTest, default_load_callback_real_single)
 
 TEST(rocfft_UnitTest, default_load_callback_real_double)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(2048,
                        1,
                        rocfft_transform_type_real_forward,
@@ -447,6 +486,13 @@ TEST(rocfft_UnitTest, default_load_callback_real_double)
 
 TEST(rocfft_UnitTest, default_store_callback_complex_single)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(256,
                        1,
                        rocfft_transform_type_complex_forward,
@@ -457,6 +503,13 @@ TEST(rocfft_UnitTest, default_store_callback_complex_single)
 
 TEST(rocfft_UnitTest, default_store_callback_complex_double)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(512,
                        1,
                        rocfft_transform_type_complex_forward,
@@ -467,6 +520,13 @@ TEST(rocfft_UnitTest, default_store_callback_complex_double)
 
 TEST(rocfft_UnitTest, default_store_callback_real_single)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(1024,
                        1,
                        rocfft_transform_type_real_forward,
@@ -477,6 +537,13 @@ TEST(rocfft_UnitTest, default_store_callback_real_single)
 
 TEST(rocfft_UnitTest, default_store_callback_real_double)
 {
+    if(hash_prob(random_seed, ::testing::UnitTest::GetInstance()->current_test_info()->name())
+       > unittest_prob)
+    {
+        GTEST_SKIP();
+    }
+
+    TEST_CALLBACK_CHECK_ASAN;
     Test_Callback test(2048,
                        1,
                        rocfft_transform_type_real_forward,
@@ -484,3 +551,4 @@ TEST(rocfft_UnitTest, default_store_callback_real_double)
                        DefaultCallbackType::STORE,
                        8);
 }
+#endif
