@@ -225,8 +225,10 @@ void build_stockham_function_pool(CompileQueue& queue)
     // build everything in the 64k LDS function pool
     function_pool fp(65536);
 
-    // fused Bluestein kernels are always built at runtime
+    // fused Bluestein and partial-pass kernels are always built at runtime
     auto fuseBlue = BluesteinFuseType::BFT_NONE;
+    auto ppType   = PartialPassType::PPT_NONE;
+    auto ppParams = StockhamPartialPassParams();
 
     for(const auto& i : fp.get_map())
     {
@@ -253,18 +255,18 @@ void build_stockham_function_pool(CompileQueue& queue)
         stockham_combo(
             scheme,
             i.second,
-            [=, &queue](int                     direction,
-                        rocfft_result_placement placement,
-                        rocfft_array_type       inArrayType,
-                        rocfft_array_type       outArrayType,
-                        EmbeddedType            ebtype,
-                        SBRC_TRANSPOSE_TYPE     sbrc_trans_type,
-                        DirectRegType           dir_reg_type,
-                        IntrinsicAccessType     intrinsic,
-                        int                     ltwd_base,
-                        int                     ltwd_step,
-                        bool                    unitstride,
-                        CallbackType            cbtype) {
+            [=, &queue, &specs](int                     direction,
+                                rocfft_result_placement placement,
+                                rocfft_array_type       inArrayType,
+                                rocfft_array_type       outArrayType,
+                                EmbeddedType            ebtype,
+                                SBRC_TRANSPOSE_TYPE     sbrc_trans_type,
+                                DirectRegType           dir_reg_type,
+                                IntrinsicAccessType     intrinsic,
+                                int                     ltwd_base,
+                                int                     ltwd_step,
+                                bool                    unitstride,
+                                CallbackType            cbtype) {
                 // intrinsic mode require non-callback and enable dir_reg
                 if((cbtype != CallbackType::NONE || dir_reg_type == FORCE_OFF_OR_NOT_SUPPORT)
                    && (intrinsic != IntrinsicAccessType::DISABLE_BOTH))
@@ -279,6 +281,7 @@ void build_stockham_function_pool(CompileQueue& queue)
                     if((scheme == CS_KERNEL_STOCKHAM && !unitstride) || length1D % 2 != 0)
                         return;
                 }
+                specs.ebtype = ebtype;
 
                 auto kernel_name = stockham_rtc_kernel_name(specs,
                                                             specs,
@@ -292,26 +295,20 @@ void build_stockham_function_pool(CompileQueue& queue)
                                                             ltwd_base,
                                                             ltwd_step,
                                                             false,
-                                                            ebtype,
                                                             dir_reg_type,
                                                             intrinsic,
                                                             sbrc_trans_type,
                                                             cbtype,
                                                             fuseBlue,
+                                                            ppType,
+                                                            ppParams,
                                                             {},
                                                             {});
                 std::function<std::string(const std::string&)> generate_src
                     = [=](const std::string& kernel_name) -> std::string {
-                    StockhamGeneratorSpecs specs{factors,
-                                                 {},
-                                                 {static_cast<unsigned int>(precision)},
-                                                 static_cast<unsigned int>(i.second.workgroup_size),
-                                                 PrintScheme(scheme)};
-                    specs.threads_per_transform = i.second.threads_per_transform[0];
-                    specs.half_lds              = i.second.half_lds;
-                    specs.direct_to_from_reg    = i.second.direct_to_from_reg;
                     return stockham_rtc(specs,
                                         specs,
+                                        ppParams,
                                         nullptr,
                                         kernel_name,
                                         scheme,
@@ -324,12 +321,12 @@ void build_stockham_function_pool(CompileQueue& queue)
                                         ltwd_base,
                                         ltwd_step,
                                         false,
-                                        ebtype,
                                         dir_reg_type,
                                         intrinsic,
                                         sbrc_trans_type,
                                         cbtype,
                                         fuseBlue,
+                                        ppType,
                                         {},
                                         {});
                 };
@@ -616,8 +613,10 @@ void build_solution_kernels(CompileQueue& queue)
     std::vector<SolutionNode> kernel_nodes;
     solmap.get_all_kernels(kernel_nodes, true);
 
-    // fused Bluestein kernels are always built at runtime
+    // fused Bluestein and partial-pass kernels are always built at runtime
     auto fuseBlue = BluesteinFuseType::BFT_NONE;
+    auto ppType   = PartialPassType::PPT_NONE;
+    auto ppParams = StockhamPartialPassParams();
 
     for(const SolutionNode& kernel_sol : kernel_nodes)
     {
@@ -674,6 +673,7 @@ void build_solution_kernels(CompileQueue& queue)
                 // kernel_sol should specify the static_dim, need to set here,
                 // so move specs to local instead of captured (need mutable if captured)
                 specs.static_dim = static_dim;
+                specs.ebtype     = ebtype;
 
                 auto kernel_name = stockham_rtc_kernel_name(specs,
                                                             specs,
@@ -687,12 +687,13 @@ void build_solution_kernels(CompileQueue& queue)
                                                             ltwd_base,
                                                             ltwd_step,
                                                             false,
-                                                            ebtype,
                                                             dir_reg_type,
                                                             intrinsic,
                                                             sbrc_trans_type,
                                                             cbtype,
                                                             fuseBlue,
+                                                            ppType,
+                                                            ppParams,
                                                             {},
                                                             {});
 
@@ -700,6 +701,7 @@ void build_solution_kernels(CompileQueue& queue)
                     = [=](const std::string& kernel_name) -> std::string {
                     return stockham_rtc(specs,
                                         specs,
+                                        ppParams,
                                         nullptr,
                                         kernel_name,
                                         scheme,
@@ -712,12 +714,12 @@ void build_solution_kernels(CompileQueue& queue)
                                         ltwd_base,
                                         ltwd_step,
                                         false,
-                                        ebtype,
                                         dir_reg_type,
                                         intrinsic,
                                         sbrc_trans_type,
                                         cbtype,
                                         fuseBlue,
+                                        ppType,
                                         {},
                                         {});
                 };
