@@ -104,44 +104,59 @@ FMKey LeafNode::GetKernelKey() const
 
 void LeafNode::GetKernelFactors()
 {
-    FMKey key     = GetKernelKey();
-    kernelFactors = pool.get_kernel(key).factors;
-
-    // Hard-coded kernel factors for len 64x64x64 partial-pass
-    // TODO: Remove this hard-coded logic once
-    // partial-pass is integrated into the stockham generators.
-    if(scheme == CS_KERNEL_STOCKHAM && applyPartialPass)
-        kernelFactors = {8, 8};
-    if(scheme == CS_KERNEL_STOCKHAM_BLOCK_CC && applyPartialPass)
-        kernelFactors = {8, 8};
+    auto kernel   = GetKernel();
+    kernelFactors = kernel.factors;
 }
 
 void LeafNode::GetKernelPartialPassFactors()
 {
-    // Hard-coded kernel partial-pass factors for len 64x64x64.
-    // TODO: Remove this hard-coded logic once
-    // partial-pass is integrated into the Stockham generators.
-    if(scheme == CS_KERNEL_STOCKHAM && applyPartialPass)
+    auto kernel     = GetKernel();
+    kernelFactorsPP = std::vector<size_t>(kernel.pp_params.pp_factors_curr.begin(),
+                                          kernel.pp_params.pp_factors_curr.end());
+
+    switch(ppOffDim)
     {
-        kernelFactorsPP = {16};
-        std::stringstream msg;
-        msg << "work in the off-dimension:" << std::endl;
-        msg << "\t     radix: [";
-        for(const auto factor : kernelFactorsPP)
-            msg << " " << factor;
-        msg << " ] pass(es) + Hadamard product with twiddle factors. \n";
-        comments.push_back(msg.str());
+    case 0: // work along x will be split between y and z
+    {
+        throw std::runtime_error(
+            "GetKernelPartialPassFactors: partial-passes along x not currently supported");
+        break;
     }
-    if(scheme == CS_KERNEL_STOCKHAM_BLOCK_CC && applyPartialPass)
+    case 1: // work along y will be split between x and z
     {
-        kernelFactorsPP = {4};
-        std::stringstream msg;
-        msg << "work in the off-dimension:" << std::endl;
-        msg << "\t     local data transposition + radix: [";
-        for(const auto factor : kernelFactorsPP)
-            msg << " " << factor;
-        msg << " ] pass(es). \n";
-        comments.push_back(msg.str());
+        if(scheme == CS_KERNEL_STOCKHAM_PP)
+        {
+            std::stringstream msg;
+            msg << "work in the off-dimension:" << std::endl;
+            msg << "\t     radix: [";
+            for(const auto factor : kernelFactorsPP)
+                msg << " " << factor;
+            msg << " ] pass(es) + Hadamard product with twiddle factors. \n";
+            comments.push_back(msg.str());
+        }
+        if(scheme == CS_KERNEL_STOCKHAM_PP_BLOCK_CC)
+        {
+            std::stringstream msg;
+            msg << "work in the off-dimension:" << std::endl;
+            msg << "\t     local data transposition + radix: [";
+            for(const auto factor : kernelFactorsPP)
+                msg << " " << factor;
+            msg << " ] pass(es). \n";
+            comments.push_back(msg.str());
+        }
+
+        break;
+    }
+    case 2: // work along z will be split between x and y
+    {
+        // x row fft + partial pass along z
+        // partial pass along z + y col fft
+        throw std::runtime_error(
+            "GetKernelPartialPassFactors: partial-passes along z not currently supported");
+        break;
+    }
+    default:
+        throw std::runtime_error("Invalid off-dimension for partial pass");
     }
 }
 
@@ -197,24 +212,18 @@ bool LeafNode::KernelCheck(std::vector<FMKey>& kernel_keys)
     // get the final key and check if we have the kernel.
     // Note that the check is trivial if we are using "specified_key"
     // since we definitly have the kernel, but not trivial if it's the auto-gen key
-    FMKey key = GetKernelKey();
-    if(!pool.has_function(key))
-    {
-        if(LOG_TRACE_ENABLED())
-            (*LogSingleton::GetInstance().GetTraceOS()) << PrintMissingKernelInfo(key);
-
+    if(!HasKernel())
         return false;
-    }
-
-    dir2regMode = (pool.get_kernel(key).direct_to_from_reg)
-                      ? DirectRegType::TRY_ENABLE_IF_SUPPORT
-                      : DirectRegType::FORCE_OFF_OR_NOT_SUPPORT;
 
     GetKernelFactors();
 
-    if(applyPartialPass)
+    if(isPartialPassEnabled())
         GetKernelPartialPassFactors();
 
+    auto kernel = GetKernel();
+    dir2regMode = (kernel.direct_to_from_reg) ? DirectRegType::TRY_ENABLE_IF_SUPPORT
+                                              : DirectRegType::FORCE_OFF_OR_NOT_SUPPORT;
+
     return true;
 }
 
@@ -289,8 +298,6 @@ void LeafNode::SetupGridParam(GridParam& gp)
     {
         if(pool.has_function(key))
         {
-            auto kernel = pool.get_kernel(key);
-
             // NB:
             // Special case on specific arch:
             // For some cases using hald_lds, finer tuning(enlarge) dynamic
@@ -304,6 +311,7 @@ void LeafNode::SetupGridParam(GridParam& gp)
                 double_half_lds_alloc = true;
             }
 
+            auto kernel = pool.get_kernel(key);
             if(kernel.half_lds && (!double_half_lds_alloc))
                 gp.lds_bytes /= 2;
         }
@@ -318,12 +326,14 @@ void LeafNode::SetupGridParam(GridParam& gp)
             if(kernel.half_lds)
                 gp.lds_bytes /= 2;
         }
+    }
 
+    if(scheme == CS_KERNEL_STOCKHAM_BLOCK_CC || scheme == CS_KERNEL_STOCKHAM_PP_BLOCK_CC)
+    {
         auto apply_large_twd = (largeTwdBase > 0 && ltwdSteps > 0);
         if(apply_large_twd && largeTwdBase < 8)
         {
             // append twiddle table to dynamic lds
-            auto kernel = pool.get_kernel(key);
             gp.lds_bytes += twiddles_large_size;
         }
     }
@@ -900,14 +910,6 @@ void CommAllToAll::ExecuteAsync(const rocfft_plan     plan,
                                 rocfft_execution_info info,
                                 size_t                multiPlanIdx)
 {
-    // check that we have as many elems in our count/offset buffers as
-    // we have ranks
-    const size_t num_ranks = plan->get_local_comm_size();
-    if(sendOffsets.size() != num_ranks || sendCounts.size() != num_ranks
-       || recvOffsets.size() != num_ranks || recvCounts.size() != num_ranks)
-        throw std::runtime_error(
-            "CommAllToAll: number of counts/offsets does not match number of ranks");
-
     if(LOG_PLAN_ENABLED())
     {
         log_plan("CommAllToAll: deciding between MPI_Ialltoall and MPI_Ialltoallv\n");
@@ -915,16 +917,78 @@ void CommAllToAll::ExecuteAsync(const rocfft_plan     plan,
 
 #ifdef ROCFFT_MPI_ENABLE
 
-    const auto elem_size = element_size(precision, arrayType);
+    // check that we have as many elems in our count/offset buffers as
+    // we have ranks
+    size_t num_ranks;
+    int    subcomm_rank;
+    if(subcomm)
+    {
+        int tmp_num_ranks = 0;
+        MPI_Comm_size(subcomm, &tmp_num_ranks);
+        if(tmp_num_ranks < 0)
+            throw std::runtime_error("Sub-communicator size is not positive");
+        num_ranks = static_cast<size_t>(tmp_num_ranks);
+        MPI_Comm_rank(subcomm, &subcomm_rank);
+    }
+    else
+    {
+        num_ranks = plan->get_local_comm_size();
+    }
 
-    // check if uniform counts
-    auto count_matches_first = [&](size_t count) { return count == sendCounts[0]; };
-    bool uniform_counts = std::all_of(sendCounts.begin(), sendCounts.end(), count_matches_first)
-                          && std::all_of(recvCounts.begin(), recvCounts.end(), count_matches_first);
+    if(sendOffsets.size() != num_ranks || sendCounts.size() != num_ranks
+       || recvOffsets.size() != num_ranks || recvCounts.size() != num_ranks)
+    {
+        throw std::runtime_error(
+            "CommAllToAll: number of counts/offsets does not match number of ranks");
+    }
 
+    int global_rank = local_comm_rank;
+
+    const auto  elem_size = element_size(precision, arrayType);
     MPI_Request request;
 
-    if(uniform_counts)
+    if(subcomm)
+    {
+        if(LOG_PLAN_ENABLED())
+            log_plan("Using MPI_Ialltoall with sub-communicators\n");
+
+        // optimization with pencil sub-communicators
+        if(!uniform_counts)
+            throw std::runtime_error(
+                "CommAllToAll::ExecuteAsync: non-uniform counts in pencil subcomm!");
+
+        void* send_ptr = sendBuf.get(in_buffer, out_buffer, local_comm_rank);
+        void* recv_ptr = recvBuf.get(in_buffer, out_buffer, local_comm_rank);
+
+        if(!send_ptr || !recv_ptr)
+        {
+            throw std::runtime_error("Buffer pointer(s) are null!");
+        }
+
+        // in subcomm: sendCounts, recvCounts, etc are sized for num_ranks, indexed by subcomm_rank
+        const int send_count_bytes = static_cast<int>(uniform_count_inside_subcomm * elem_size);
+
+        int ret = MPI_Ialltoall(sendBuf.get(in_buffer, out_buffer, local_comm_rank),
+                                send_count_bytes,
+                                MPI_CHAR,
+                                recvBuf.get(in_buffer, out_buffer, local_comm_rank),
+                                send_count_bytes,
+                                MPI_CHAR,
+                                subcomm,
+                                &request);
+
+        if(ret != MPI_SUCCESS)
+        {
+            char errmsg[MPI_MAX_ERROR_STRING];
+            int  errlen = 0;
+            MPI_Error_string(ret, errmsg, &errlen);
+            comm_status   = COMM_MPI_ERROR;
+            error_message = "MPI_Ialltoall (subcomm) failed on global rank "
+                            + std::to_string(global_rank) + ": " + std::string(errmsg);
+            return;
+        }
+    }
+    else if(uniform_counts)
     {
         if(LOG_PLAN_ENABLED())
             log_plan("Using MPI_Ialltoall\n");
@@ -958,6 +1022,7 @@ void CommAllToAll::ExecuteAsync(const rocfft_plan     plan,
         if(LOG_PLAN_ENABLED())
             log_plan("Using MPI_Ialltoallv\n");
 
+        // non-uniform exchange case (default)
         const int local_comm_rank = plan->get_local_comm_rank();
 
         // MPI takes ints for everything, convert our size_t elements to int bytes
